/*
 * Copyright (c) 2003 Sun Microsystems, Inc. All Rights Reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 * - Redistribution of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 * - Redistribution in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 * Neither the name of Sun Microsystems, Inc. or the names of
 * contributors may be used to endorse or promote products derived from
 * this software without specific prior written permission.
 *
 * This software is provided "AS IS," without a warranty of any kind. ALL
 * EXPRESS OR IMPLIED CONDITIONS, REPRESENTATIONS AND WARRANTIES,
 * INCLUDING ANY IMPLIED WARRANTY OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE OR NON-INFRINGEMENT, ARE HEREBY EXCLUDED. SUN
 * MICROSYSTEMS, INC. ("SUN") AND ITS LICENSORS SHALL NOT BE LIABLE FOR
 * ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
 * DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES. IN NO EVENT WILL SUN OR
 * ITS LICENSORS BE LIABLE FOR ANY LOST REVENUE, PROFIT OR DATA, OR FOR
 * DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL, INCIDENTAL OR PUNITIVE
 * DAMAGES, HOWEVER CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY,
 * ARISING OUT OF THE USE OF OR INABILITY TO USE THIS SOFTWARE, EVEN IF
 * SUN HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
 *
 * You acknowledge that this software is not designed or intended for use
 * in the design, construction, operation or maintenance of any nuclear
 * facility.
 *
 * Sun gratefully acknowledges that this software was originally authored
 * and developed by Kenneth Bradley Russell and Christopher John Kline.
 */

package org.j3d.opengl.swt.internal.ri.x11;

// External imports
import javax.media.opengl.GLContext;
import javax.media.opengl.GLException;

import com.sun.opengl.impl.GLContextShareSet;
import com.sun.opengl.impl.x11.GLX;

// Local Imports
// None


public class X11ExternalGLContext extends X11GLContext
{
    private boolean firstMakeCurrent = true;
    private boolean created = true;

    public X11ExternalGLContext()
    {
        super(null, null);
        context = GLX.glXGetCurrentContext();
        drawable = new Drawable(GLX.glXGetCurrentDisplay());
        GLContextShareSet.contextCreated(this);
        resetGLFunctionAvailability();
    }

    protected void create()
    {
    }

    protected int makeCurrentImpl() throws GLException
    {
        mostRecentDisplay = drawable.getDisplay();
        if(firstMakeCurrent)
        {
            firstMakeCurrent = false;
            return CONTEXT_CURRENT_NEW;
        }

        return CONTEXT_CURRENT;
    }

    protected void releaseImpl() throws GLException
    {
    }

    protected void destroyImpl() throws GLException
    {
        created = false;
        GLContextShareSet.contextDestroyed(this);
    }

    public boolean isCreated()
    {
        return created;
    }

    // Need to provide the display connection to extension querying APIs
    class Drawable extends X11GLDrawable
    {
        Drawable(long display)
        {
            super(null, null);
            this.display = display;
        }

        public GLContext createContext(GLContext shareWith)
        {
            throw new GLException("Should not call this");
        }

        public int getWidth()
        {
            throw new GLException("Should not call this");
        }

        public int getHeight()
        {
            throw new GLException("Should not call this");
        }

        public void setSize(int width, int height)
        {
            throw new GLException("Should not call this");
        }
    }
}
